/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * ================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://forceten.tidalwave.it
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id$
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.location;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import java.util.List;
import it.tidalwave.netbeans.util.NotFoundException;

/***********************************************************************************************************************
 *
 * This class represents the ordered set of ancestors of a given {@link GeoLocation}.
 *
 * @author Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public interface GeoPath extends Iterable<GeoLocation>
  {
    public final static Class<GeoPath> GeoPath = GeoPath.class;

    /*******************************************************************************************************************
     *
     * Returns the size of this path.
     *
     * @return  the size
     *
     ******************************************************************************************************************/
    @Nonnegative
    public int size();

    /*******************************************************************************************************************
     *
     * Returns the item at the given position in the path.
     *
     * @param  index  the position
     * @return        the element
     *
     ******************************************************************************************************************/
    @Nonnull
    public GeoLocation get (@Nonnegative int index);

    /*******************************************************************************************************************
     *
     * Finds the first ancestor of the given type according to the given {@link GeoSchema}.
     *
     * @param   geoSchema   the schema
     * @param   type        the type
     * @return              the ancestor
     * @throws              {@link NotFoundException} if none is found
     *
     ******************************************************************************************************************/
    @Nonnull
    public GeoLocation findByType (@Nonnull GeoSchema schema,
                                   @Nonnull Object type)
      throws NotFoundException;
    
    /*******************************************************************************************************************
     *
     * Returns the current path as a {@link List}.
     *
     * @return  the list
     *
     ******************************************************************************************************************/
    @Nonnull
    public List<GeoLocation> asList();
  }
